import * as font from './font';
export declare type LineCapStyle = 'BUTT' | 'ROUND' | 'SQUARE';
export declare type RuleValue = "even-odd" | "evenodd" | "non-zero" | "nonzero";
export interface DrawOpts {
    fill?: boolean;
    stroke?: boolean;
}
export interface ScaleOpts {
    origin?: {
        x: number;
        y: number;
    };
}
export interface TextOptions {
    width?: number;
    align?: 'right' | 'center' | string;
    characterSpacing?: number;
}
export declare class Page {
    width: number;
    height: number;
    objectNumber: number | undefined;
    buffer: string;
    constructor(width: number, height: number);
}
export declare class FPdf {
    /**
     * The version of the PDF spec that we are targeting. The minimum
     * version that supports the features we support should be used
     *
     * @type {String}
     */
    readonly pdfVersion: string;
    /**
     * PDF documents contain elements called objects. They are numbered.
     * This is to keep track of the current number
     *
     * @type {Number}
     */
    private _currentObjectNumber;
    /**
     * The font key of the last font that was outputted to the buffer
     * Any new BT/ET commands will default to using this font
     */
    private _currentFontKey;
    /**
     * The font key of the last font that was selected by calling code
     * This is what we use to track the latest selected font.
     * The command to use this font may not have been written to the file yet
     */
    private _chosenFontKey;
    private _currentFontSize;
    private _chosenFontSize;
    private _pages;
    private _objects;
    private _fonts;
    private _pen;
    private _metadata;
    private _buffer;
    private _subset;
    private _standardPageSizes;
    private readonly _currentPage;
    private readonly _chosenFont;
    readonly currentFontMetrics: {
        ascender: number;
        descender: number;
        gap: number;
        lineHeight: number;
    };
    close(): void;
    addPage(size?: any, orientation?: any, rotation?: any): void;
    strokeColor(red: number, green: number, blue: number): void;
    getTextWidth(text: string, opts?: TextOptions): number;
    getTextHeight(): number;
    drawOptsToPdfOp(opts?: DrawOpts): string;
    /**
     * Send a command to the PDF to fill in the current path
     */
    fill(): FPdf;
    /**
     * Send a command to the PDF to stroke the current path
     */
    stroke(): FPdf;
    /**
     * Send a command to the PDF to stroke the current path
     */
    fillAndStroke(): FPdf;
    clip(rule?: RuleValue): void;
    /**
     * Stream out a description of a rectangle to the PDF. Nothing will actually be drawn until
     * a fill or stroke command goes out after it
     * @param x The x coordinate to draw at
     * @param y The y coordiante to draw at
     * @param w The width of the rectangle
     * @param h The height of the rectangle
     */
    rect(x: number, y: number, w: number, h: number): FPdf;
    /**
     * Stream out a description of a rounded rectangle to the PDF. Nothing will actually be drawn until
     * a fill or stroke command goes out after it
     * @param x The x coordinate to draw at
     * @param y The y coordiante to draw at
     * @param w The width of the rectangle
     * @param h The height of the rectangle
     */
    roundedRect(x: number, y: number, w: number, h: number, r?: number): FPdf;
    /**
     * Draw a rect immediately. Don't wait for a fill or stroke command
     * @param x The x coordinate to draw at
     * @param y The y coordiante to draw at
     * @param w The width of the rectangle
     * @param h The height of the rectangle
     * @param opts Whether to fill or stoke or both
     */
    drawRect(x: number, y: number, w: number, h: number, opts?: DrawOpts): FPdf;
    moveTo(x: number, y: number): FPdf;
    lineTo(x: number, y: number): FPdf;
    /**
     * Transform given coordinates and draw a bezier curve with them

     * @param {number} cp1x handle control point 1 x coordinate
     * @param {number} cp1y handle control point 1 y coordinate
     * @param {number} cp2x handle control point 2 x coordinate
     * @param {number} cp2y handle control point 2 y coordinate
     * @param {number} x    final vector point x coordinate
     * @param {number} y    final vector point y coordinate
     */
    bezierCurveTo(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;
    /**
     * Modify the current transformation matrix
     * @param {number} m11 can be used for scaling the x axis or for rotations
     * @param {number} m12 can be used for skew or for rotation
     * @param {number} m21 can be used for skew or for rotation
     * @param {number} m22 can be used for scaling the y axis or for rotations
     * @param {number} dx  moves the x coordinate of the origin
     * @param {number} dy  moves the y coordinate of the origin
     */
    transform(m11: number, m12: number, m21: number, m22: number, dx: number, dy: number): void;
    scale(scalex: number, scaley?: number | ScaleOpts, options?: ScaleOpts): void;
    lineWidth(width: number): void;
    lineCap(style: LineCapStyle): void;
    closePath(): void;
    setCharacterSpacing(advanceWidth: number): void;
    save(): void;
    restore(): void;
    /**
     * move the pen to (x, y)
     * @param {number} x the horizontal coordinate to move the pen to
     * @param {number} y the vertical coordinate to move the pen to
     */
    $m(x: number, y: number): void;
    /**
     * draw a line from the current pen positon to to (x, y)
     * @param {number} x the horizontal coordinate to draw the line to
     * @param {number} y the vertical coordinate to draw the line to
     */
    $l(x: number, y: number): void;
    $W(rule?: RuleValue): void;
    /**
     * End the path object without filling or stroking it. This operator is a “path-painting no-op,”
     * used primarily for the side effect of changing the current clipping path
     */
    $n(): void;
    /**
     * draw a bezier curve with the given coordinates
     *
     * @param {number} cp1x handle control point 1 x coordinate
     * @param {number} cp1y handle control point 1 y coordinate
     * @param {number} cp2x handle control point 2 x coordinate
     * @param {number} cp2y handle control point 2 y coordinate
     * @param {number} x    final vector point x coordinate
     * @param {number} y    final vector point y coordinate
     */
    $c(cp1x: number, cp1y: number, cp2x: number, cp2y: number, x: number, y: number): void;
    /**
     * Modify the current transformation matrix
     *
     * @param {number} cp1x [description]
     * @param {number} cp1y [description]
     * @param {number} cp2x [description]
     * @param {number} cp2y [description]
     * @param {number} x    [description]
     * @param {number} y    [description]
     */
    $cm(m11: number, m12: number, m21: number, m22: number, dx: number, dy: number): void;
    $h(): void;
    /**
     * save the current graphics state
     */
    $q(): void;
    /**
     * restore the previous graphics state
     */
    $Q(): void;
    /**
     * stroke the current path (made with moveTo, lineTo, etc)
     */
    $S(): void;
    /**
     * set the LineCap style
     * @param {LineCapStyle} style [description]
     */
    $J(style: LineCapStyle): void;
    /**
     * sets the current line width
     * @param {number} width the width to use in points for the pen
     */
    $w(width: number): void;
    /**
     * use charachter spacing with the given advance width
     * @param {number} advanceWidth the amount to advance the pen after each character
     */
    $Tc(advanceWidth: number): void;
    $rect(x: number, y: number, w: number, h: number): void;
    $fillStroke(opts?: DrawOpts): void;
    $RG(red: number, green: number, blue: number): void;
    /**
     * add a custom TrueType or OpenType font to this PDF
     * the font will be embedded into the PDf file
     *
     * @param {string}    family  [description]
     * @param {string}    style   [description]
     * @param {font.Font} newFont [description]
     */
    addCustomFont(family: string, style: string, newFont: font.Font): void;
    addStandardFont(family: font.StandardFontFamilyName, style?: string, newFont?: font.Font): void;
    private addFont(fontKey, newFont);
    setFont(family: string, style?: string, size?: number): void;
    setFontSize(size: number): void;
    _encodeText(s: string): string;
    /**
     * Write text to the page
     *
     * FIXME: We are doing a lot of _putToCurrentPage here
     *        We should update it to use $functions and
     *        then appropriately named wrappers
     *
     * @param {number}                     x
     * @param {number}                     y
     * @param {string}                     text
     * @param {TextOptions =    {}}        opts
     */
    text(x: number, y: number, text: string, opts?: TextOptions): void;
    output(filename?: string): void;
    readonly buffer: string;
    readonly finalBuffer: string;
    /**
     * The native PDF coordinate system sets the origin at the bottom left corner of the page
     * It is more intuitive to construct PDFs by starting with the top as y = 0 with postitive y
     * values moving you down the page. This method transforms coordinates from the convient form
     * to the PDF native form
     *
     * @param {number} x The x coordinate
     * @param {number} y The y coordinate (At the top of the page y=0. Moving down the page y gets larger)
     */
    private _transformPoint(x, y);
    private _getFontKey(family, style);
    private _getpagesize(size);
    private _beginpage(size?, orientation?, rotation?);
    /**
     * Begins a new object
     */
    private _newobj(objectNumber?);
    private _getoffset();
    /**
     * _put actually appends the string to the buffer
     */
    private _put(s);
    private _putToCurrentPage(s);
    private _putToPage(s, pageNumber);
    private _putresources();
    private _putresourcedict();
    private _putinfo();
    private _formatDate(date);
    private _enddoc();
    private _putcatalog();
    private _putheader();
    private _puttrailer();
    private _putpages();
    private _putpage(page);
    private _putfonts();
    private _putstreamobject(data);
    private _putstream(data);
}
